local L = LibStub("AceLocale-3.0"):GetLocale("Guild_Channel", true)

function Guild_Channel:debug(text)
	if Guild_Channel_debug then
		DEFAULT_CHAT_FRAME:AddMessage("|cff00ff9aGC:|r " .. text, 1, 0, 0)
	end
end

local function options_OnShow(frame)
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	title:SetText("Guild Channel")

	local subtitle = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subtitle:SetHeight(16)
	subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subtitle:SetPoint("RIGHT", parent, -32, 0)
	subtitle:SetNonSpaceWrap(true)
	subtitle:SetJustifyH("LEFT")
	subtitle:SetJustifyV("TOP")
	subtitle:SetText(L["notes"])
	
	minimapicon = CreateFrame("CheckButton", "Guild_Channel_MinimapIconCheck", frame, nil)
	minimapicon:SetWidth(25)
	minimapicon:SetHeight(25)
	minimapicon:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", 0, -8)
	minimapicon:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up") 
	minimapicon:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down") 
	minimapicon:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight", "ADD") 
	minimapicon:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check") 
	minimapicon:SetChecked(not Guild_Channel_Icon.hide)
	minimapicon:SetScript("OnClick", function()
		if Guild_Channel_Icon.hide then
			Guild_Channel_MinimapIcon:Show("Guild_Channel_MinimapIcon")
			Guild_Channel_Icon.hide = false
		else
			Guild_Channel_MinimapIcon:Hide("Guild_Channel_MinimapIcon")
			Guild_Channel_Icon.hide = true
		end
	end)
	minimapicon:Show() 

	title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	title:SetPoint("LEFT", minimapicon, "RIGHT", 0, 0)
	title:SetText(L["Show minimap icon"])

	title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	title:SetPoint("TOPLEFT", minimapicon, "BOTTOMLEFT", 0, -8)
	title:SetText("Version: |cff9999ff" ..  GetAddOnMetadata("Guild_Channel", "Version"))
	

	frame:SetScript("OnShow", nil)
end

local options = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
options.name = "Guild Channel"
options:Hide()
options:SetScript("OnShow", options_OnShow)
InterfaceOptions_AddCategory(options)

local function DropdownInitFunction()

-- level 2
if _G["UIDROPDOWNMENU_MENU_LEVEL"] == 2 then
	if _G["UIDROPDOWNMENU_MENU_VALUE"] == L["Sort"] then
		info = {}
		info.text = L["Sort"]
		info.notClickable = 1
		info.isTitle = 1
		UIDropDownMenu_AddButton(info, 2)
		info = {}
		info.text = L["Name"]
		info.checked = function() if Guild_Channel_Config.sort == "NAME" then return true else return false end end
		info.func = function() Guild_Channel_Config.sort = "NAME" end
		UIDropDownMenu_AddButton(info, 2)
		info = {}
		info.text = L["Class"]
		info.checked = function() if Guild_Channel_Config.sort == "CLASS" then return true else return false end end
		info.func = function() Guild_Channel_Config.sort = "CLASS" end
		UIDropDownMenu_AddButton(info, 2)
	end
	if _G["UIDROPDOWNMENU_MENU_VALUE"] == L["Set name for alts"] then
		info = {}
		info.text = L["Set name for alts"]
		info.notClickable = 1
		info.isTitle = 1
		UIDropDownMenu_AddButton(info, 2)
		table.foreach(Guild_Channel.PlayerList, function(k,v)
			if (v.rank == 3) then
				info = {}
				info.text = k
				info.func = function() Guild_Channel:ChangeAltPopup(k) end
				UIDropDownMenu_AddButton(info, 2)
			end
		end)
	end
return
end

-- level 1
info = {}
info.text = "Guild Channel |cff00ffff"  .. GetAddOnMetadata("Guild_Channel", "Version")
info.notClickable = 1
info.isTitle = 1
UIDropDownMenu_AddButton(info)

local alts = 0

table.foreach(Guild_Channel.PlayerList, function(k,v)
	if (v.rank == 3) then
		alts = alts + 1
	end
end)

if alts > 0 then
	info = {}
	info.text = L["Set name for alts"]
	info.value = L["Set name for alts"]
	info.hasArrow = 1
	UIDropDownMenu_AddButton(info)
end

info = {}
info.text = L["Sort"]
info.value = L["Sort"]
info.hasArrow = 1
UIDropDownMenu_AddButton(info)
if (Guild_Channel_Config.main ~= "") then
	info = {}
	if (Guild_Channel_Config.chan == "") then
		info.text = L["Set channel"]
	else
		info.text = L["Channel: "] .. _G["GREEN_FONT_COLOR_CODE"] .. Guild_Channel_Config.chan
	end
	info.func = function() Guild_Channel:ChangeChannelPopup() end
	UIDropDownMenu_AddButton(info)
	info = {}
	if (Guild_Channel_Config.guild == "") then
		info.text = L["Set guild"]
	else
		info.text = L["Main guild: "] .. _G["GREEN_FONT_COLOR_CODE"] .. "<" .. Guild_Channel_Config.guild .. ">"
	end
	info.func = function() Guild_Channel:ChangeGuildPopup() end
	UIDropDownMenu_AddButton(info)
end
info = {}
if (Guild_Channel_Config.main == "") then
	info.text = L["Set as main char"]
else
	info.text = L["Main char: "] .. Guild_Channel:ClassColor(Guild_Channel_Config.main, L[Guild_Channel_Config.class])
end
info.func = function() Guild_Channel:SetMainPopup() end
UIDropDownMenu_AddButton(info)

end

UIDropDownMenu_Initialize(Guild_Channel_Menu, DropdownInitFunction, "MENU")

function Guild_Channel:ChangeChannelPopup()
StaticPopupDialogs["Guild_Channel_ChangeChannel"] = {
	text = _G["NORMAL_FONT_COLOR_CODE"] .. "Guild Channel\n\n" .. L["Set channel"] .. "\n",
	button1 = _G["ACCEPT"],
	button2 = _G["CANCEL"],
	hasEditBox = 1,
	OnAccept = function(self)
		Guild_Channel:LeaveCmdChannel()
		Guild_Channel_Config.chan = self.editBox:GetText()
		Guild_Channel.CHAN_CMD = Guild_Channel_Config.chan.."users"
		Guild_Channel:EnableChannel()
	end,
	OnShow = function(self)
		self.editBox:SetFocus()
		self.editBox:Insert(Guild_Channel_Config.chan)
	end,
	OnHide = function(self)
		self.editBox:SetText("")
	end,
	EditBoxOnEnterPressed = function(self)
		local parent = self:GetParent()
		Guild_Channel:LeaveCmdChannel()
		Guild_Channel_Config.chan = parent.editBox:GetText()
		Guild_Channel.CHAN_CMD = Guild_Channel_Config.chan.."users"
		Guild_Channel:EnableChannel()
		parent:Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}
StaticPopup_Show("Guild_Channel_ChangeChannel")
end

function Guild_Channel:ChangeGuildPopup()
StaticPopupDialogs["Guild_Channel_ChangeGuild"] = {
	text = _G["NORMAL_FONT_COLOR_CODE"] .. "Guild Channel\n\n" .. L["Type the guild name of the guild that uses the cannel"] .. "\n",
	button1 = _G["ACCEPT"],
	button2 = _G["CANCEL"],
	hasEditBox = 1,
	OnAccept = function(self)
		Guild_Channel_Config.guild = self.editBox:GetText()
		Guild_Channel:LeaveCmdChannel()
		Guild_Channel:EnableChannel()
	end,
	OnShow = function(self)
		self.editBox:SetFocus()
		self.editBox:Insert(Guild_Channel_Config.guild)
	end,
	OnHide = function(self)
		self.editBox:SetText("")
	end,
	EditBoxOnEnterPressed = function(self)
		local parent = self:GetParent()
		Guild_Channel_Config.guild = parent.editBox:GetText()
		Guild_Channel:LeaveCmdChannel()
		Guild_Channel:EnableChannel()
		parent:Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}
StaticPopup_Show("Guild_Channel_ChangeGuild")
end

function Guild_Channel:ChangeAltPopup(alt)
StaticPopupDialogs["Guild_Channel_ChangeAlt"] = {
	text = _G["NORMAL_FONT_COLOR_CODE"] .. "Guild Channel\n\n" .. L["Type a name for "] .. alt .."\n",
	button1 = _G["ACCEPT"],
	button2 = _G["CANCEL"],
	hasEditBox = 1,
	OnAccept = function(self)
		Guild_Channel_Config.T[alt] = self.editBox:GetText()
		Guild_Channel:SendCmdMsg("U#" .. alt .. "#" .. Guild_Channel_Config.T[alt])
	end,
	OnShow = function(self)
		self.editBox:SetFocus()
		if Guild_Channel_Config.T[alt] then self.editBox:Insert(Guild_Channel_Config.T[alt]) end
	end,
	OnHide = function(self)
		self.editBox:SetText("")
	end,
	EditBoxOnEnterPressed = function(self)
		local parent = self:GetParent()
		Guild_Channel_Config.T[alt] = parent.editBox:GetText()
		Guild_Channel:SendCmdMsg("U#" .. alt .. "#" .. Guild_Channel_Config.T[alt])
		parent:Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}
StaticPopup_Show("Guild_Channel_ChangeAlt")
end

function Guild_Channel:SetMainPopup()
StaticPopupDialogs["Guild_Channel_SetMain"] = {
	text = _G["NORMAL_FONT_COLOR_CODE"] .. "Guild Channel\n\n" .. L["Is "] .. select(1, UnitName("player")) .. L[" your main char?"] .. "\n",
	button1 = _G["ACCEPT"],
	button2 = _G["CANCEL"],
	OnAccept = function(self)
		Guild_Channel:LeaveCmdChannel()
		local player, class, _ = UnitName("player")
		_, class = UnitClass("player");
		Guild_Channel_Config.main = player
		Guild_Channel_Config.class = class
		if IsInGuild() then Guild_Channel_Config.guild = GetGuildInfo("player") end
		Guild_Channel:EnableChannel()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}
StaticPopup_Show("Guild_Channel_SetMain")
end
