Guild_Channel = CreateFrame("Frame", "Guild_Channel")
Guild_Channel_Menu = CreateFrame("Frame", "Guild_Channel_Menu", UIParent, "UIDropDownMenuTemplate")
Guild_Channel_Config = { guild = "", chan = "", main = "", class = "", sort = "NAME", T = {} }
Guild_Channel_Icon = {}

Guild_Channel_debug = false

local tooltip = LibStub("LibQTip-1.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Guild_Channel", true)
LibStub("AceTimer-3.0"):Embed(Guild_Channel)
LibStub:GetLibrary("LibWho-2.0"):Embed(Guild_Channel)
Guild_Channel.ldb = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Guild_Channel", {
	type = "data source",
	text = "GC",
	icon = "Interface\\Addons\\Guild_Channel\\icon.tga"
})

Guild_Channel:RegisterEvent("ADDON_LOADED")
Guild_Channel:RegisterEvent("CHAT_MSG_CHANNEL")
Guild_Channel:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")
Guild_Channel:RegisterEvent("CHAT_MSG_CHANNEL_LIST")
Guild_Channel:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
Guild_Channel:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE")
Guild_Channel:RegisterEvent("GUILD_ROSTER_UPDATE")
Guild_Channel:RegisterEvent("PARTY_MEMBERS_CHANGED")
Guild_Channel:RegisterEvent("PLAYER_LEVEL_UP")
Guild_Channel:RegisterEvent("GUILD_MOTD")

Guild_Channel:SetScript("OnEvent", function(_, event, ...)
	Guild_Channel[event](Guild_Channel, ...)
end)

local _G = getfenv(0)
local t0 = nil
local totaltime = 0
local colortable = {}
local RAID_CLASS_COLORS = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS
for class, color in pairs(RAID_CLASS_COLORS) do
     colortable[class] = string.format("%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255)
end

for k, v in next, LOCALIZED_CLASS_NAMES_MALE do	L[v] = k end
for k, v in next, LOCALIZED_CLASS_NAMES_FEMALE do L[v] = k end
for k, v in next, RAID_CLASS_COLORS do L[k] = k end

Guild_Channel.tt = nil
Guild_Channel.count = 0
Guild_Channel.lastQ = 0
Guild_Channel.motd = nil
Guild_Channel.mod = ""

Guild_Channel.chan_joined = false
Guild_Channel.chan_init = nil
Guild_Channel.chan_init_count = 0

Guild_Channel.PlayerList = {}
Guild_Channel.newPlayer = true

local function hidett(self, elapsed)
	totaltime = totaltime + elapsed
	if totaltime > 0.2 then
		totaltime = 0
		if Guild_Channel.tt and not MouseIsOver(Guild_Channel.tt) then
			Guild_Channel.tt:SetScript("OnEnter", nil)
			Guild_Channel.tt:SetScript("OnLeave", nil)	
			tooltip:Release(Guild_Channel.tt)
			Guild_Channel.tt = nil
			Guild_Channel:SetScript("OnUpdate", nil)
		elseif not Guild_Channel.tt then
			Guild_Channel:SetScript("OnUpdate", nil)
			return
		end
	end
end

-- ### LDB ###

function Guild_Channel.ldb.OnEnter(self)
	Guild_Channel:debug("OnEnter")
	if IsInGuild() then GuildRoster() end
	Guild_Channel:SendWho()
	Guild_Channel.tt = tooltip:Acquire("Guild_Channel_Tooltip", 5, "LEFT", "LEFT", "LEFT", "LEFT", "CENTER")
	Guild_Channel.tt:EnableMouse(true)
	Guild_Channel.tt:SetScript("OnEnter", function(self) Guild_Channel:SetScript("OnUpdate", nil) end)
	Guild_Channel.tt:SetScript("OnLeave", function(self) Guild_Channel:SetScript("OnUpdate", hidett) end)
	Guild_Channel.tt:SmartAnchorTo(self)
	Guild_Channel.tt:SetScale(1)
	Guild_Channel.tt:SetBackdropBorderColor(1, 1, 1, 1)
	Guild_Channel:DrawTooltip()	
	Guild_Channel.tt:Show()
end

function Guild_Channel.ldb.OnLeave(self)
	if Guild_Channel.tt and Guild_Channel.tt:IsVisible() then
		Guild_Channel:SetScript("OnUpdate", hidett)
	end
end

function Guild_Channel.ldb.OnClick(self, button)
	if Guild_Channel.tt then
		Guild_Channel.tt:SetScript("OnEnter", nil)
		Guild_Channel.tt:SetScript("OnLeave", nil)
		tooltip:Release(Guild_Channel.tt)
		Guild_Channel.tt = nil
	end
	if button == "RightButton" then
		ToggleFriendsFrame(4)
	else
		ToggleDropDownMenu(1, nil, Guild_Channel_Menu, self, -20, 0)
	end
end

-- ### Tooltip ###

local function EditMOTD()
	if Guild_Channel.tt then
		Guild_Channel.tt:SetScript("OnEnter", nil)
		Guild_Channel.tt:SetScript("OnLeave", nil)
		tooltip:Release(Guild_Channel.tt)
		Guild_Channel.tt = nil
	end
	GuildMOTDEditButton:Click()
end

local function ClickHandlerFunc(cell, arg1, button)
	if (arg1 and button == "LeftButton") then
		if IsAltKeyDown() then
			InviteUnit(arg1)
		else
			SetItemRef(string.format("player:%s", arg1), string.format("|Hplayer:%s|h[%s|h", arg1, arg1), "LeftButton")
		end
	end
end

local function Cyan(text) return "|cff00ffff" .. text .. "|r" end

Guild_Channel.sort = {
	NAME =	function(a, b)
				return t0[a].name < t0[b].name
			end,
	CLASS =	function(a, b)
				if t0[a].class < t0[b].class then
					return true
				elseif t0[a].class > t0[b].class then
					return false
				else
					return Guild_Channel.sort.NAME(a, b)
				end
			end,
}

function Guild_Channel:ClassColor(text, class)
	return "|cff".. colortable[L[class]] .. text .. "|r"
end

function Guild_Channel:DrawTooltip()
	Guild_Channel:GetRoster()
	Guild_Channel:debug("DrawTooltip")
	Guild_Channel.tt:Clear()
	
	if (Guild_Channel_Config.main == "") then
		local player, class, _ = UnitName("player")
		_, class = UnitClass("player");
		Guild_Channel.tt:AddHeader("Guild Channel")
		Guild_Channel.tt:AddLine(_G["NORMAL_FONT_COLOR_CODE"] .. L["No main char set."])
		Guild_Channel.tt:AddLine(L["Click on the Guild Channel minimap icon\nif "] .. Guild_Channel:ClassColor(player, L[class]) .. L[" is your main char, to set the main char."])
		return
	end
	if (Guild_Channel_Config.chan == "") then
		Guild_Channel.tt:AddHeader("Guild Channel")
		Guild_Channel.tt:AddLine(_G["NORMAL_FONT_COLOR_CODE"] .. L["No chat channel set."])
		Guild_Channel.tt:AddLine(L["Click on the Guild Channel minimap icon to set a chat channel."])
		return
	end
	if (Guild_Channel.count == 0 and Guild_Channel.chan_joined == false) then
		Guild_Channel.tt:AddHeader("Guild Channel")
		Guild_Channel.tt:AddLine(L["You are currently not in channel "] .. _G["GREEN_FONT_COLOR_CODE"] .. Guild_Channel_Config.chan)
		Guild_Channel.tt:AddLine(L["Type "] .. _G["NORMAL_FONT_COLOR_CODE"] .. "/join " .. Guild_Channel_Config.chan .. " |cffffffff" .. L["to join the channel."] .. "|r")
		return
	end
	if (Guild_Channel.count == 0 and Guild_Channel.chan_joined == true) then
		Guild_Channel.tt:AddHeader("Guild Channel")
		Guild_Channel.tt:AddLine(L["Please wait..."])
		return
	end
	
	if Guild_Channel.newPlayer then
		Guild_Channel:debug("request player names")
		table.foreach(Guild_Channel.PlayerList, function(k,v)
			if (v.rank == 3 and Guild_Channel_Config.T[k] == nil) then
				Guild_Channel:SendCmdMsg("W#" .. k)
			end
		end)
		Guild_Channel.newPlayer = false
	end
	
	local line, party, zone, _zone, alt
	local c1, c2, c3 = 0, 0, 0

	zone = GetRealZoneText()
	if Guild_Channel.motd then
		line = Guild_Channel.tt:AddHeader("")
		Guild_Channel.tt:SetCell(line, 1, "MOTD", nil, "LEFT", 5)
		Guild_Channel.tt:AddLine("")
		Guild_Channel.tt:AddLine("")
	end

	t0 = {}
	local t1 = {}
	table.foreach(Guild_Channel.PlayerList, function(k,v)
		t0[k] = {}
		t0[k].name = k
		t0[k].class = (v.class == nil) and "" or v.class
		if v.rank == 1 then c1 = c1 + 1 end
		if v.rank == 2 then c2 = c2 + 1 end
		if v.rank == 3 then c3 = c3 + 1 end
		table.insert(t1, k)
	end)
	
	table.sort(t1, Guild_Channel.sort[Guild_Channel_Config.sort])

	-- Mains
	if (c1 > 0) then Guild_Channel.tt:AddLine("") Guild_Channel.tt:AddLine("") end
	table.foreach(t1,
		function (k,v)
			if Guild_Channel.PlayerList[v].rank == 1 then
				party = (UnitInParty(v) or UnitInRaid(v)) and "\226\128\162" or ""
				if (zone == Guild_Channel.PlayerList[v].zone and Guild_Channel.PlayerList[v].zone) then _zone = _G["GREEN_FONT_COLOR_CODE"] .. zone else _zone = Guild_Channel.PlayerList[v].zone or "" end
				line = Guild_Channel.tt:AddLine(party, Guild_Channel:ClassColor(v .. "  ", Guild_Channel.PlayerList[v].class), L["Main"] .. "  ", _zone .. "  ", Guild_Channel.PlayerList[v].level)
				Guild_Channel.tt:SetLineScript(line, "OnMouseUp", ClickHandlerFunc, v)
			end
		end
	)
	
	-- Alts
	if (c2 > 0) then Guild_Channel.tt:AddLine("") Guild_Channel.tt:AddLine("") end
	table.foreach(t1,
		function (k,v)
			if Guild_Channel.PlayerList[v].rank == 2 then
				party = (UnitInParty(v) or UnitInRaid(v)) and "\226\128\162" or ""
				line = Guild_Channel.tt:AddLine(party, Guild_Channel:ClassColor(v .. "  ", Guild_Channel.PlayerList[v].class), Guild_Channel.PlayerList[v].main .. "  ", Guild_Channel.PlayerList[v].zone .. "  ", Guild_Channel.PlayerList[v].level)
				Guild_Channel.tt:SetLineScript(line, "OnMouseUp", ClickHandlerFunc, v)
			end
		end
	)

	-- Unknowns
	if (c3 > 0) then Guild_Channel.tt:AddLine("") Guild_Channel.tt:AddLine("") end
	table.foreach(t1,
		function (k,v)
			if Guild_Channel.PlayerList[v].rank == 3 then
				party = (UnitInParty(v) or UnitInRaid(v)) and "\226\128\162" or ""
				alt = Guild_Channel_Config.T[v] or "?"
				line = Guild_Channel.tt:AddLine(party, _G["NORMAL_FONT_COLOR_CODE"] .. v .. "  ", alt .. "  ")
				Guild_Channel.tt:SetLineScript(line, "OnMouseUp", ClickHandlerFunc, v)
			end
		end
	)
	
	local width = Guild_Channel.tt:GetWidth()

	if Guild_Channel.motd then
		Guild_Channel.tt:SetCell(2, 1, Cyan(Guild_Channel.motd or ""), nil, "LEFT", 5, nil, nil, nil, width)
		if CanEditMOTD() and Guild_Channel_Config.guild == GetGuildInfo("player") then 
			Guild_Channel.tt:SetLineScript(2, "OnMouseUp", EditMOTD)
		end
	end

	line = Guild_Channel.tt:AddLine("")
	Guild_Channel.tt:SetCell(line, 1, _G["GREEN_FONT_COLOR_CODE"] .. L["\nClick a player to whisper to him.\nAlt-click a player to invite him to your party."], nil, "LEFT", 5, nil, nil, nil, width)
	t0 = nil
end

-- ### Guild Roster ###

function Guild_Channel:GUILD_ROSTER_UPDATE()
	Guild_Channel:debug("GUILD_ROSTER_UPDATE")
	Guild_Channel:GetRoster()
	Guild_Channel:Update()
end

function Guild_Channel:GetRoster()
	if (GetGuildInfo("player") == Guild_Channel_Config.guild) then
		Guild_Channel.motd = GetGuildRosterMOTD()
		for i=1, GetNumGuildMembers(true) do
			local name, rank, _, level, class, zone, note, offnote, connected, status = GetGuildRosterInfo(i)
			if (Guild_Channel.PlayerList[name]) then
				Guild_Channel.PlayerList[name].rank = 1
				Guild_Channel.PlayerList[name].class = class
				Guild_Channel.PlayerList[name].zone = zone
				Guild_Channel.PlayerList[name].level = level
				Guild_Channel_Config.T[name] = name
			end
		end
	end
end

-- ### Who Query ###

function Guild_Channel:SendWho()
	if (GetGuildInfo("player") ~= Guild_Channel_Config.guild) then
		Guild_Channel:debug("SendWho")
		if (GetTime() > Guild_Channel.lastQ + 5) then
			Guild_Channel.lastQ = GetTime()
			Guild_Channel:Who(Guild_Channel_Config.guild, {
				queue = WHOLIB_QUEUE_QUIET, 
				callback = GCWhoCallback
			})
		end
	end
end

function GCWhoCallback(query, results, complete, name)
	Guild_Channel:debug("WhoCallback")
	for _,result in pairs(results) do
		if (result.Guild == Guild_Channel_Config.guild) then
			if (Guild_Channel.PlayerList[result.Name]) then
				Guild_Channel.PlayerList[result.Name].rank = 1
				Guild_Channel.PlayerList[result.Name].class = result.Class
				Guild_Channel.PlayerList[result.Name].zone = result.Zone
				Guild_Channel.PlayerList[result.Name].level = result.Level
				Guild_Channel_Config.T[result.Name] = result.Name
			end
		end
	end
	Guild_Channel:Update()
end

function Guild_Channel:PARTY_MEMBERS_CHANGED()
	Guild_Channel:Update()
end

function Guild_Channel:PLAYER_LEVEL_UP(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	Guild_Channel:SendPosition(arg1)
end

function Guild_Channel:GUILD_MOTD(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if Guild_Channel_Config.guild == GetGuildInfo("player") then
		Guild_Channel.motd = arg1
		Guild_Channel:SendCmdMsg("M#" .. arg1)
	end
end

function Guild_Channel:ADDON_LOADED(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if (arg1 == "Guild_Channel") then
		Guild_Channel.CHAN_CMD = Guild_Channel_Config.chan.."users"
		Guild_Channel.chan_init = Guild_Channel:ScheduleRepeatingTimer(function () Guild_Channel:ListChannel() end, 1)
		Guild_Channel:UnregisterEvent("ADDON_LOADED")
		Guild_Channel.ADDON_LOADED = nil
		Guild_Channel_MinimapIcon = LibStub("LibDBIcon-1.0")
		Guild_Channel_MinimapIcon:Register("Guild_Channel_MinimapIcon", Guild_Channel.ldb, Guild_Channel_Icon)
	end
end

function Guild_Channel:ListChannel()
	if Guild_Channel.chan_init_count > 30 then
		Guild_Channel:CancelTimer(Guild_Channel.chan_init, true)
		Guild_Channel:debug("giving up on chan")
		return
	end
	Guild_Channel.chan_init_count = Guild_Channel.chan_init_count + 1
	local id, _ = GetChannelName(Guild_Channel_Config.chan)
	if (id > 0) then
		Guild_Channel:CancelTimer(Guild_Channel.chan_init, true)
		Guild_Channel:debug("channel found!")
		if Guild_Channel.chan_joined then return end -- return if joined
		Guild_Channel:EnableChannel() -- uireload -> list
		return
	end
	Guild_Channel:debug("no channel found")
end